//package toast;

import java.lang.*;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import javax.swing.JScrollPane.*;
import java.util.*;

public class NetObject {
   private int sizeX;
   private int sizeY;


  // variables for Net Data from the SNNS-File:
   private String generatedAt;
   private String networkName;
   private String sourceFiles;
   private int noOfUnits;
   private int noOfConnections;
   private String noOfUnitTypes;
   private String noOfSiteTypes;
   private String learningFunction;
   private String pruningFunction;
   private String ffLearningFunction;
   private String updateFunction;


   // Strings for parsing the SNNS-File:
   final String GENERATED_AT = "generated at";
   // LUKE: append other Keywords !!!!
   final String UNIT_SECTION_TITLE = "unit definition section";
   final String CONNECTION_SECTION_TITLE = "connection definition section";
   final String NO_OF_UNITS = "no. of units :";
   final String NO_OF_CONNECTIONS = "no. of connections :";


   // Variables containing the ANN-Structure
   final int MAX_UNITS = 1000;
   Unit units[] = new Unit[MAX_UNITS];
   private int unitCount = 0;
   final int MAX_CONNECTIONS = 11000;
   Connection connections[] = new Connection[MAX_CONNECTIONS];
   private int connectionCount = 0;


   public void save (String netFileName) {
      Properties netProps = new Properties();
      Properties unitProps = new Properties();
      String key;
      netFileName = netFileName+".props";

      for (int i = 0; i < unitCount; i++) {
          try {
              unitProps = units[i].getProperties(unitProps);
	      for (Enumeration e = unitProps.propertyNames() ; e.hasMoreElements() ;) {
                  key = (String)e.nextElement();
		  netProps.setProperty("unit"+i+key,unitProps.getProperty(key));
              }

          } catch (java.lang.NullPointerException e) {
              System.out.println("Exeption at copying Properties from  Unit " + i);
              break;
          }
      }

      try {
          FileOutputStream out = new FileOutputStream(netFileName);
          netProps.store(out, "---"+netFileName+"---");
          out.close();
      } catch (IOException e) {}


   } 

   public void readPropsFile(String netFileName) {
       boolean error = false;
       netFileName = netFileName+".props";
       Properties netProps = new Properties();
       Properties unitProps = new Properties();
       String key;
       String header;
       int headerLen;

       try {
           FileInputStream in = new FileInputStream(netFileName);
           netProps.load(in);
           in.close();
       }
       catch  (FileNotFoundException e) {
	   error = true;
       }
       catch  (IOException e) {
	   error = true;
       }

       if (!error) {
	   for (int i = 0; i < unitCount; i++) {
	       header = "unit"+i;	
               headerLen = header.length();	
	       for (Enumeration e = netProps.propertyNames() ; e.hasMoreElements() ;) {
                   key = (String)e.nextElement();
		   if (key.startsWith(header)) {
                      unitProps.setProperty(key.substring(headerLen),netProps.getProperty(key));
	//	      System.out.println("copying netProperties: "+key);     
		   }
               }
	       units[i].setX(Integer.parseInt(unitProps.getProperty("positionx")));
               units[i].setY(Integer.parseInt(unitProps.getProperty("positiony")));
	       units[i].setActivation(unitProps.getProperty("activated").equals("true"));
	       units[i].setProperties(unitProps);
	   }


       }
  }



   public int readSNNSFile(File f) throws java.io.IOException{
      java.io.FileInputStream fi;
      java.io.BufferedReader in;
      String line;
      String nextline;

      // check File if we can use it (we need to write it if you want to change it!):
      if ( !f.isFile()) {
        return -1;
      }

      if ( !f.canRead()) {
        return -1;
      }
      if ( !f.canWrite()) {
        return -1;
      }

      //ok... now we can open a input stream:

      fi = new FileInputStream(f);
      in = new BufferedReader( new InputStreamReader(fi));

      //Parse Lines:

      int linenr = 0;
      do {
	 linenr++;
         line = in.readLine();
         if (line == null) break;
         if (line.startsWith(GENERATED_AT)) {
            //System.out.println(" found GENERATED_AT in line "+linenr);
	    try {
              generatedAt = line.substring(GENERATED_AT.length()+1,line.length());
            } catch (StringIndexOutOfBoundsException e) {
              generatedAt = "";
            }
         } else if (line.startsWith(NO_OF_UNITS)) {
	    //System.out.println(" found NO_OF_UNITS in line "+linenr);
            try {
              noOfUnits = Integer.parseInt(line.substring(NO_OF_UNITS.length()+1).trim());
            } catch (StringIndexOutOfBoundsException e) {
              noOfUnits = 0;
            }
         } else if (line.startsWith(NO_OF_CONNECTIONS)) {
	    //System.out.println(" found NO_OF_CONNECTIONS in line "+linenr);
            try {
              noOfConnections = Integer.parseInt(line.substring(NO_OF_CONNECTIONS.length()+1).trim());
            } catch (StringIndexOutOfBoundsException e) {
              noOfConnections = 0;
            }
        } else if (line.startsWith(UNIT_SECTION_TITLE)) {
            // the unit definiton section: search for the line which contains the string "No." at the beginning.
            // two lines later there should be the line containing the first Unit data.
	    //System.out.println(" found UNIT_SECTION_TITLE in line "+linenr);
            do {
		linenr++;
               line = in.readLine();
            } while (!line.startsWith("no."));
	    //System.out.println(", found header");
	    linenr++;
            line = in.readLine();   // read line without important data...
            // read the unit-definition lines
	    // read first line from section, the next lines will be read in the 'append lines' loop
            linenr++;
            nextline = in.readLine();
            for ( unitCount = 0; unitCount<noOfUnits; unitCount++) {
	      //System.out.println("    unit "+unitCount+" starts at line " + linenr);
              line = nextline; 
	      // append lines which belong to line and store first line of the next line in 'nextline'
	      do {
                  linenr++;
                  nextline = in.readLine();
                  if (nextline.startsWith("    ")) {
                      // next line belongs to line !
                      line = line + nextline;
                  } else {
                      break;
                  }
              } while (true);

              //parse line-String
              units[unitCount] = new Unit(line);
            }
	    //System.out.println(" done UNIT_SECTION.\n");
         } else if (line.startsWith(CONNECTION_SECTION_TITLE)) {
            // the connection definiton section:
            // search for the line which contains the string "target" at the beginning.
            // two lines later there should be the line containing the first Unit data.
	    //System.out.println(" found CONNECTION_SECTION_TITLE in line "+linenr);
            connectionCount = 0;

            do {
	       linenr++;
               line = in.readLine();
            } while (!line.startsWith("target"));
	    linenr++;
            line = in.readLine();   // read line without important data...
            // read the connection-definition lines
            // break loop if the line contains a "-" at the beginning

	    linenr++;
	    nextline = in.readLine();
            do {
	      //System.out.println(" parsing CONNECTION-LINE : line "+linenr);
	      line = nextline;
              if (line.startsWith("-")) break;

              //get the target
              int target = Integer.parseInt(line.substring(1,7).trim());
              String site = line.substring(line.indexOf("|")+1,line.lastIndexOf("|"));

              //System.out.println("target : "+target);
              //System.out.println("    '"+line+"'");
	      // look if there are some other lines to append and at 
	      // least cur header and append a ","
	      do {
		  linenr++;
 	          nextline = in.readLine();
	          if (nextline.startsWith("       ")) {
		      // next line belongs to line !
		      line = line + nextline;
                  } else {
		      break;
	          }
	      } while (true);
	      line = line.substring(line.lastIndexOf("|")+1) + ",";
	      //System.out.println("    '"+line+"'");

              int i = 0;
              while ((i = line.indexOf(",")) != -1) {
               //   System.out.println("          '"+line.substring(0,i)+"'");
                  String s = line.substring(0,i);
                  int divider = s.indexOf(":");
                  int source = Integer.parseInt(s.substring(0,divider).trim());
                  String weight = s.substring(divider+1).trim();
                  //System.out.println("Connection "+connectionCount+"     '"+source+":"+weight+"'");
                  line = line.substring(i+1);
                  connections[connectionCount] = new Connection(units[target-1], units[source-1], weight, site);
                  ++connectionCount;
              }
            } while (true);
	    //System.out.println(" done CONNECTION_SECTION.\n");
        }

         // TODO LUKE: Append all other header keywords for complete parsing of SNNS-Files.


      }while (true);

      //System.out.println(" ended parsing without obvious error\n");
      return 0;
   }

   /** toString returns a String containing the basis Information of the String
   * use this for debugging purpose
   * @returns String
   */
   public String toString() {
     int i;
     String s = new String("NetObject Debugging Information:\n");
     s = s + "  " + GENERATED_AT + " '" + generatedAt + "'\n";
     s = s + "  " + NO_OF_UNITS + " '" + noOfUnits + "'\n";
     s = s + "  " + NO_OF_CONNECTIONS + " '" + noOfConnections + "'\n";
/*     for (i=0; i<noOfUnits; i++) {
       s = s+"     Unit["+i+"] : "+units[i].toString();
     }
     for (i=0; i<noOfConnections; i++) {
       //       System.out.println(i);
       s = s+"     Connection["+i+"] : " + connections[i].toString();
     }
*/
     return s;

   }

   public void paintPS(PSGr g, Properties p) {
        for (int i=0; i<noOfConnections; i++) {
            try {
                connections[i].paintPS(g, p);
            } catch (java.lang.NullPointerException e) {
                System.out.println("Exeption at drawing Connection " + i);
                break;
            }
        }
        for (int i = 0; i < unitCount; i++) {
            try {
                units[i].paintPS(g, p);
            } catch (java.lang.NullPointerException e) {
                System.out.println("Exeption at drawing Unit " + i);
                break;
            }
        }
   }





   /** this is the redraw-function for the net itself.
    * it invokes the paint-method of the Units and Connections included in the net.
    * @param g Graphics-Object which determines where to draw.
    * @param thumbnail if true a thumbnail is drawn.
    * @param P Properties with general drawing-properties for the net.
    */
   public void paintComponent(Graphics g, boolean thumbnail, Properties p) {

      if (!thumbnail) {
         for (int i=0; i<noOfConnections; i++) {
	   try {
              connections[i].paint(g, p);
	   } catch (java.lang.NullPointerException e) {
	      System.out.println("Exeption at drawing Connection " + i);	
              break;
           }
         }
         for (int i = 0; i < unitCount; i++) {
	    try {
                units[i].paint(g, p);
	    } catch (java.lang.NullPointerException e) {
		System.out.println("Exeption at drawing Unit " + i);
	        break;
	    }
         }
      } 
   }

   public void paintThumbnail(Graphics g) {
       for (int i=0; i<noOfConnections; i++) {
          try {
              connections[i].paintThumbnail(g);
          } catch (java.lang.NullPointerException e) {
              System.out.println("Exeption at drawing Connection " + i);        
              break;
          }
       }
       for (int i = 0; i < unitCount; i++) {
           try {
               units[i].paintThumbnail(g);
           } catch (java.lang.NullPointerException e) {
               System.out.println("Exeption at drawing Unit " + i);
               break;
           }
       }
   }
   

   public void setSize(int sizeX, int sizeY) {


       Rectangle r = getBorder();
       if (r!=null) {
           for (int i = 0; i < unitCount; i++) {
               units[i].translate(-r.x,-r.y);
               units[i].translate( (units[i].getX()*(sizeX-r.width))/r.width , (units[i].getY()*(sizeY-r.height))/r.height );
           }
       }

   }

   public void scaleTo(int x, int y) {
        Point p;
	double minX = 1000;
	double maxX = 0;
	double minY = 1000;
	double maxY = 0;

	for (int i = 0; i < unitCount; i++) {
           try {
               p = units[i].getLocation();
	   }
	   catch (java.lang.NullPointerException e) {
		break;
	   }

	   minX = Math.min(minX, p.getX());
	   maxX = Math.max(maxX, p.getX());

	   minY = Math.min(minY, p.getY());
           maxY = Math.max(maxY, p.getY());
	}

	double w = maxX-minX;
	double h = maxY-minY;

	float fx = (float) w / (float) x ;
	float fy = (float) h / (float) y ;

	for (int i = 0; i < unitCount; i++) {
	   units[i].scale(minX,minY,fx,fy);
        }
   }



   public Rectangle getBorder() {
      try {
            Rectangle r = units[0].getBorder();
            for (int i = 1; i < unitCount; i++) {
                r.add(units[i].getBorder());
            }
            return r;
      } catch (NullPointerException e) {
            return null;
      }

   }

   public void translate(int dx, int dy) {
      for (int i = 0; i < noOfUnits; i++) {
	  try {
              units[i].translate(dx, dy);
	  } catch (NullPointerException e) {}
      }
   }

   /** this method returns the Object which is visible and which includes the point p
    *  at the moment only the Unit-Objects will be returned, Connections are not tested
    */
   public Object getObject(Point p) {
       Unit hitUnit;
       for (int i = 0; i < unitCount; i++) {
           hitUnit = units[i].getUnit(p);
           if (hitUnit != null) {
               return hitUnit;
           }
       }

       return null;
       // append also tests for other Objects (i.e. connections)
/*       Connection hitConnection;
       for (int i = 0; i < connectionCount; i++) {
	   hitConnection = connections[i].getConnection(p);
	   if (hitConnection != null) {
		return hitConnection;
	   }	
       }
       return null;

*/
   }

   public int getUnitCount() {
	return unitCount;
   }

   public Unit getUnit(int i) {
      return units[i];
   }



/*
   public boolean MouseClicked(java.awt.event.MouseEvent evt) {
      boolean handled = false;
      for (int i = 0; i < unitCount; i++) {
         handled = units[i].MouseClicked(evt);
         if (handled) break;
      }
      return handled;
   }
*/
   public void main(java.lang.String[] args) {
   }

}

