//package toast;

import java.lang.*;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import javax.swing.JScrollPane.*;

/**
 * The NetProperties Class extend a JFrame with Components for changing the Properties of the
 * graphical appearance of a Neuronal Network.
 *
 * @author Andreas Lackner
 * @version 0.1
 */
public class NetProperties extends javax.swing.JFrame {

  private java.util.Properties props = new java.util.Properties();
  private String PropertiesFileName;

  final JColorChooser colorChooser = new JColorChooser();

  // variables declaration for GUI components
  private JPanel panel;
  private JPanel buttonPanel;

  private javax.swing.JTextField distanceTextField;
  private javax.swing.JComboBox gridLayoutComboBox;
  private javax.swing.JCheckBox snapCheckBox;
  private JButton bgColorChooserButton;
  private JButton gridColorChooserButton;
  private javax.swing.JLabel jLabel1;
  private javax.swing.JLabel jLabel2;
  private javax.swing.JLabel jLabel3;
  private javax.swing.JLabel jLabel4;
  private javax.swing.JLabel jLabel5;


  private JButton okButton;
  private JButton resetToDefaultButton;
  private JButton cancleButton;

  // End of variables declaration


  /** Initializes the Form.
   * It loads The Properties from a given File. At the Moment there is no error-handling !
   * Be sure to give a valid FileName !
   */
  public NetProperties(String FileName) {
    this.PropertiesFileName = FileName;
    initComponents ();
    loadProperties (FileName);
    setProperties();
    initListeners ();
    pack ();
    setVisible (false);
  }

  /**
   * returns the Properties - field
   * @see java.util.Properties
   */
  public java.util.Properties getProperties() {
    return props;
  }

  private void loadProperties(String FileName) {
    try {
        FileInputStream in = new FileInputStream(FileName);
        props.load(in);
        in.close();
    }
    catch  (FileNotFoundException e) {}
    catch  (IOException e) {}
  }

  private void setProperties() {
    bgColorChooserButton.setBackground(new Color(Integer.parseInt(props.getProperty("BackGroundRed","204"))
                                              ,Integer.parseInt(props.getProperty("BackGroundGreen","204"))
                                              ,Integer.parseInt(props.getProperty("BackGroundBlue","204"))));
    gridColorChooserButton.setBackground(new Color(Integer.parseInt(props.getProperty("GridRed","0"))
                                              ,Integer.parseInt(props.getProperty("GridGreen","0"))
                                              ,Integer.parseInt(props.getProperty("GridBlue","0"))));


    snapCheckBox.setSelected(props.getProperty("GridSnap","true").equals("true"));
    distanceTextField.setText(props.getProperty("GridDistance","22"));
    gridLayoutComboBox.setSelectedIndex(Integer.parseInt(props.getProperty("GridLayout","1")));

  }

  /** This method is called from within the constructor to
   * initialize the form.
   */
  private void initComponents () {
    setResizable (false);
    setTitle ("Network Visualization Properties");
    panel = new JPanel();
    buttonPanel = new JPanel();



    okButton = new JButton("Ok");
    cancleButton = new JButton("Cancle");
    resetToDefaultButton = new JButton("Default");


    buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
    buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
    buttonPanel.add(Box.createHorizontalGlue());
    buttonPanel.add(resetToDefaultButton);
    buttonPanel.add(Box.createRigidArea(new Dimension(10, 0)));
    buttonPanel.add(cancleButton);
    buttonPanel.add(Box.createRigidArea(new Dimension(10, 0)));
    buttonPanel.add(okButton);

    panel.setLayout(new GridLayout(5,2,10,10));
    panel.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));

    getContentPane ().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));

    getContentPane ().add (Box.createRigidArea(new Dimension(0, 16)));
    getContentPane ().add (panel);
    getContentPane ().add (Box.createRigidArea(new Dimension(0, 5)));
    getContentPane ().add (buttonPanel);


    jLabel1 = new javax.swing.JLabel ();
    jLabel1.setText ("Grid Distance");
    //getContentPane ().add (jLabel1);
    panel.add (jLabel1);

    distanceTextField = new javax.swing.JTextField ();
    distanceTextField.setPreferredSize (new java.awt.Dimension(40, 21));
    distanceTextField.setMaximumSize (new java.awt.Dimension(40, 21));
    distanceTextField.setMinimumSize (new java.awt.Dimension(40, 21));

    //getContentPane ().add (distanceTextField);
    panel.add (distanceTextField);


    jLabel2 = new javax.swing.JLabel ();
    jLabel2.setText ("Grid Layout :");
    panel.add (jLabel2);

    gridLayoutComboBox = new javax.swing.JComboBox ();
    gridLayoutComboBox.addItem("invisible");
    gridLayoutComboBox.addItem("dots");
    gridLayoutComboBox.addItem("lines");
    panel.add (gridLayoutComboBox);

    jLabel3 = new javax.swing.JLabel ();
    jLabel3.setText ("Snap Objects to grid");
    panel.add (jLabel3);

    snapCheckBox = new javax.swing.JCheckBox ();
    snapCheckBox.setActionCommand ("snapCheckBoxActionCommand");
    panel.add (snapCheckBox);

    jLabel4 = new javax.swing.JLabel ("Background-Color");
  //  jLabel4.setText ("Background-Color");
    panel.add (jLabel4);

    bgColorChooserButton = new JButton();
    panel.add (bgColorChooserButton);

    jLabel5 = new javax.swing.JLabel ("Grid-Color");
    panel.add (jLabel5);

    gridColorChooserButton = new JButton();
    panel.add (gridColorChooserButton);

  }

  private void initListeners() {
    this.addWindowListener (new java.awt.event.WindowAdapter () {
        public void windowClosing (java.awt.event.WindowEvent evt) {
          exitForm ();
        }
      }
    );
    cancleButton.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          loadProperties(PropertiesFileName);
          setProperties();
          exitForm ();
        }
      }
    );
    okButton.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          saveProperties ();
          exitForm();
        }
      }
    );
    resetToDefaultButton.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          loadProperties ("NetProperties.props.default");
          setProperties ();
        }
      }
    );
    distanceTextField.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          propertiesChanged (evt);
        }
      }
    );
    bgColorChooserButton.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          bgColorChooserShow ();
        }
      }
    );
    gridColorChooserButton.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          gridColorChooserShow ();
        }
      }
    );
    gridLayoutComboBox.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          propertiesChanged (evt);
        }
     }
    );
    snapCheckBox.addActionListener (new java.awt.event.ActionListener () {
        public void actionPerformed (java.awt.event.ActionEvent evt) {
          propertiesChanged (evt);
        }
      }
    );
  }


  private void propertiesChanged (java.awt.event.ActionEvent evt) {
    //check Values if they are valid
    try {
      int i = Integer.parseInt(distanceTextField.getText().trim());
    }
    catch (NumberFormatException e) {
      distanceTextField.setText(props.getProperty("GridDistance"));
    }

    // Save all values to the Properties Object.
    Object oldVal = props.setProperty("GridLayout", String.valueOf(gridLayoutComboBox.getSelectedIndex()));
    oldVal = props.setProperty("GridDistance", distanceTextField.getText().trim() );
    oldVal = props.setProperty("GridSnap", String.valueOf(snapCheckBox.isSelected()) );

  }

  private void saveProperties() {
    try {
      FileOutputStream out = new FileOutputStream(PropertiesFileName);
      props.store(out, "---NetProperties.Props---");
      out.close();
    }
    catch (IOException e) {}

  }

  private void bgColorChooserShow () {
      Color c = new Color(0);
      c = colorChooser.showDialog(this, "select background-color",bgColorChooserButton.getBackground());
      if (c != null) {
         bgColorChooserButton.setBackground(c);
         //System.out.println(colorChooserButton.getBackground().toString());
         Object oldVal = props.setProperty("BackGroundRed",String.valueOf(bgColorChooserButton.getBackground().getRed()));
         oldVal = props.setProperty("BackGroundGreen",String.valueOf(bgColorChooserButton.getBackground().getGreen()));
         oldVal = props.setProperty("BackGroundBlue",String.valueOf(bgColorChooserButton.getBackground().getBlue()));
      }
  }

  private void gridColorChooserShow () {
      Color c = new Color(0);
      c = colorChooser.showDialog(this, "select grid-color",gridColorChooserButton.getBackground());
      if (c != null) {
         gridColorChooserButton.setBackground(c);
         //System.out.println(colorChooserButton.getBackground().toString());
         Object oldVal = props.setProperty("GridRed",String.valueOf(gridColorChooserButton.getBackground().getRed()));
         oldVal = props.setProperty("GridGreen",String.valueOf(gridColorChooserButton.getBackground().getGreen()));
         oldVal = props.setProperty("GridBlue",String.valueOf(gridColorChooserButton.getBackground().getBlue()));
      }
  }




  public String toString () {
     return props.toString();
  }

  /** hide Dialog
   */
  private void exitForm() {
    this.setVisible(false);
  }

  public static void main(java.lang.String[] args){
    NetProperties np =  new NetProperties("NetProperties.Props");
    np.show();
  }

}
