//package toast;
import java.lang.*;
import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.io.*;

public class Unit extends Object {
  //private UnitJFrame unitJFrame = new UnitJFrame();
  private int numOfDataFields = 10;
  private DataField dataField[] = new DataField[numOfDataFields];

  private int     no;
  private String  typeName;
  private String  unitName;
  private String  act;
  private String  bias;
  private String  st;
  private int     x;			// this is the Position of the Unit (the point where Connections start)
  private int     y;
  private int     z;
  private String  actfunction;
  private String  outFunction;
  private String  sitesList;

  private int thumbnailX;  
  private int thumbnailY;

  private boolean hasFocus;

  private int dataFieldNumberHitByMouse;

  final int SIZE = 10;

  private boolean activated;

  /** the void constructor */
  public Unit() {}


  /** This is the constructor for Unit. As we only parse SNNS-Files at the moment,
   * we don't need any other constructors.
   * The given String contains on lineram data String containing a line out of an SNNS-File
   * @param data String containing a line out of an SNNS-File
   */
  public Unit(String data) {

     // Tokenize the String data and create the Datafields.
     // forward the default unit-Properties to the Datafields to customize them

     int endOfSnippet;
     int startOfSnippet = 1;

     Properties unitDefaultProps=new Properties();
     unitDefaultProps = loadProperties("UnitProperties.props");

     for (int i=0; i<numOfDataFields; i++) {
         endOfSnippet = data.indexOf("|",startOfSnippet);
         if (i < numOfDataFields - 1) {
	     dataField[i] = new DataField(data.substring(startOfSnippet,endOfSnippet).trim());
         } else {
	     dataField[i]= new DataField(data.substring(startOfSnippet).trim());
         }
         startOfSnippet=endOfSnippet+1;
     }

     // now we can get the real information out of the snippets...
     // at the moment we just read the number of the Unit and it's Position
     // LUKE: Append all Data parsing

     // parsing number
     no = Integer.parseInt(dataField[0].getData());

     // parsing type name
     typeName = dataField[1].getData();

     // parsing unit name
     unitName = dataField[2].getData();

     // parsing position.
     int endOfX = dataField[6].getData().indexOf(",");  // find Position of first "," in "x,y,z"
     int endOfY = dataField[6].getData().indexOf(",",endOfX+1);   // find Position of second ","
     setLocation(Integer.parseInt(dataField[6].getData().substring(0,endOfX).trim()),
                 Integer.parseInt(dataField[6].getData().substring(endOfX+1,endOfY).trim()),
                 Integer.parseInt(dataField[6].getData().substring(endOfY+1).trim()));



     setProperties(unitDefaultProps);


  }

  public Properties getProperties(Properties p) {
     for (int i=0; i<numOfDataFields; i++) {
	// get graphical properties of every Datafield
	p = dataField[i].getProperties(p,i);
	// get vector to every Datafield (starting from Unit-Center)
        p.setProperty ("posx"+i,  String.valueOf(dataField[i].getX() - x));
        p.setProperty ("posy"+i,  String.valueOf(dataField[i].getY() - y));
     }
     p.setProperty ("activated", String.valueOf(activated));
     p.setProperty ("positionx", String.valueOf(x));
     p.setProperty ("positiony", String.valueOf(y));
     return p;
  }

  public void setProperties(Properties p) {
     for (int i=0; i<numOfDataFields; i++) {
	// set graphical properties of datafields
        dataField[i].setProperties(p,i);
	// set position of datafield, starting at Center of Unit
        dataField[i].setX(this.x);
        dataField[i].setY(this.y);
	dataField[i].translate(Integer.parseInt(p.getProperty("posx"+i)), Integer.parseInt(p.getProperty("posy"+i)));
	//System.out.println("Translating DataField "+Integer.parseInt(p.getProperty("posx"+i))+" "+Integer.parseInt(p.getProperty("posy"+i)));
     }
  }

  public void setFocus(boolean focus) {
     hasFocus = focus;
  }

  public boolean getFocus() {
     return hasFocus;
  }

  public void toggleFocus() {
     if (getFocus()){
	setFocus(false);
     } else {
	setFocus(true);
     }
  }

  /** toString returns a String containing the basis Information of the String
   * use this for debugging purpose
   * @returns String
   */
  public String toString() {
      String s = " no = '"+no+"' unitname = '"+unitName+"'  Pos = '"+x+","+y+","+z+"'\n";
      for (int i=0; i<numOfDataFields; i++) {
          s = s + "    DF "+ i + getValueText(i) +"\n";
      }
      return s;
  }

  public String getValueText(int i) {
	return dataField[i].toString();
  }

  /** this method returns the drawing location of the Unit (exactly the center of it)
   * subject to change in the future: a new parameter Properties p, where to say if its 2D or 3D
   * @param none !!! DANGER !!! WILL CHANGE !!!
   */
  public Point getLocation() {
      Point p = new Point(getX(),getY());
      return p;
  }
  public void setLocation(int x, int y, int z) {
      this.x = 40*x;
      this.y = 40*y;
      this.z = 40*z;
  }

  public void setX(int x) {
      this.x=x;
  }
  public void setY(int y) {
      this.y=y;
  }
  public int getX() {
     return  x;
  }

  public int getY() {
     return  y;
  }

  public int getThumbnailX() {
     return  thumbnailX + 5;
  }

  public int getThumbnailY() {
     return  thumbnailY + 5;
  }


  public void translate(int dx, int dy) {
      this.x += dx;
      this.y += dy;
      for (int i = 0; i < numOfDataFields; i++) {
          dataField[i].translate(dx, dy);
      }
  }

  public void scale(double x, double y, float fx, float fy) {
	thumbnailX = (int)((this.getX() - x) / fx);
	thumbnailY = (int)((this.getY() - y) / fy);
  }



  public void customTranslate(int dx, int dy) {
      if (dataFieldNumberHitByMouse == 0) {
	  this.x += dx;
          this.y += dy;
          for (int i = 0; i < numOfDataFields; i++) {
              dataField[i].customTranslate(dx, dy);
          }
      } else {
	  //System.out.println("moving Datafield by "+dx+", "+dy);
	  dataField[dataFieldNumberHitByMouse].translate(dx, dy);
      }
  }

  public void customSetLocation(int x,int y) {
     //System.out.println("DataField hit: "+dataFieldNumberHitByMouse);
     int px = dataField[dataFieldNumberHitByMouse].getX();
     int py = dataField[dataFieldNumberHitByMouse].getY();
 
     int dx = x - px; 
     int dy = y - py;
     this.customTranslate(dx,dy);
  }
    
  public Rectangle getBorder() {
      Rectangle r = new Rectangle();
      for (int i = 0; i < numOfDataFields; i++) {
         Rectangle dr = dataField[i].getBorder();
         if (dr!=null) {
             if (r.width==0) {
                 r.setBounds(dr);
             } else {
                 r.add(dr);
             }
         }
      }
      return r;
  }

  /** the paint method draws the Unit according to the parameter Properties p.
   * @param g The Graphics-Object which determines where to draw.
   * @param p Properties-Object with general Net-Drawing-Properties.
   */
  public void paint(Graphics g, Properties p) {
     //int grid = Integer.parseInt(p.getProperty("GridDistance"));

     //g.fillRect((x*grid)-(SIZE/2),(y*grid)-(SIZE/2),SIZE,SIZE);
     for (int i = 0; i < numOfDataFields; i++) {
        dataField[i].paintComponent(g, p, hasFocus);
     }
  }

  public void paintPS(PSGr g, Properties p) {
     for (int i = 0; i < numOfDataFields; i++) {
        dataField[i].paintPS(g, p);
     }
  }

  public void paintThumbnail(Graphics g) {
     g.setColor(Color.black);
     g.fillRect(getThumbnailX()-2,getThumbnailY()-2,4,4);
  }




  private Properties loadProperties(String FileName) {
    Properties p = new Properties();
    try {
        FileInputStream in = new FileInputStream(FileName);
        p.load(in);
        in.close();
    }
    catch  (FileNotFoundException e) {}
    catch  (IOException e) {}

    return p;
  }

  public Unit getUnit(Point p) {
      boolean hit = false;
      for (int i = 0; i < numOfDataFields; i++) {
         hit = dataField[i].isVisible() & dataField[i].contains(p);
         if (hit) {
            //
	    dataFieldNumberHitByMouse = i;
            return this;

         }
      }

      dataFieldNumberHitByMouse = -1;
      return null;
   }

   public int getNo() {
       return no;
   }

   public boolean isActivated() {
       return activated;
   }

   public void toggleActivation() {
	if (activated) {
	    setActivation( false );
        } else {
	    setActivation( true );
        }
   }

   public void setActivation(boolean activ) {
	this.activated = activ;
	for (int i = 0; i < numOfDataFields; i++) {
            dataField[i].setActivation(activ);
        }
   }
	
       
}
